﻿using IQAP.Ofertas.Negocio.API;
using IQAP.Ofertas.Negocio.Datos;
using IQAP.Ofertas.Negocio.Modelo;
using IQAP.Ofertas.Recursos.Textos;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace IQAP.Ofertas.Negocio
{
    public class CalculadoraPrecio
    {
        /// <summary>
        /// El coste base es SIEMPRE el coste de Material
        /// </summary>
        /// <param name="parametros"></param>
        /// <param name="costeMaterial"></param>
        /// <param name="precioRecomendado"></param>
        /// <param name="calculoValores"></param>
        /// <returns></returns>
        public DatosPrecioCalculado CalcularPrecios(ParametrosCalculoPrecio parametros, decimal costeMaterial, string warningsCosteMaterial, Dictionary<short, decimal> precioRecomendado, ConfiguracionCalculo configCalculo)
        {
            var precios = new DatosPrecioCalculado(costeMaterial);
            precios.FechaCalculo = DateTime.Now;
            precios.WarningsCosteMaterial = warningsCosteMaterial;

            //Calculo de precios recomendado y mínimo
            if(precioRecomendado!=null && precioRecomendado.Keys.Count>0)
            {
                precios.Recomendado = precioRecomendado;
                precios.EsPrecioCatalogo = true;
            }
            else
            {
                //Calcular precio recomendado
                CalcularPrecioTipoPrecio(parametros, precios, configCalculo, (short) Precios.Recomendado);
            }

            //Calcular precio minimo
            CalcularPrecioTipoPrecio(parametros, precios, configCalculo, (short)Precios.Minimo);

            //Moneda
            if (!String.IsNullOrEmpty(parametros.Moneda) && parametros.Moneda != "EUR")
            {
                AplicarCambioMoneda(precios.Coste, parametros.MonedaValorCambio);
                AplicarCambioMoneda(precios.Minimo, parametros.MonedaValorCambio);
                if (!precios.EsPrecioCatalogo)
                {
                    AplicarCambioMoneda(precios.Recomendado, parametros.MonedaValorCambio);
                }
                precios.CosteMaterial = precios.CosteMaterial * parametros.MonedaValorCambio;
            }

            return precios;
        }

        private void CalcularPrecioTipoPrecio(ParametrosCalculoPrecio parametros, DatosPrecioCalculado precios, ConfiguracionCalculo configCalculo, short tipoPrecio)
        {
            //Si hay avisos de coste no calcular
            if (String.IsNullOrEmpty(precios.WarningsCosteMaterial))
            {
                //Criterio 1: Base
                CalcularCriterioBase(parametros, precios, configCalculo, tipoPrecio);

                //Criterio 2: Polímero
                CalcularCriterioPolimero(parametros, precios, configCalculo, tipoPrecio);

                //Criterio 3: Distribuidor
                CalcularCriterioDistribuidor(parametros, precios, configCalculo, tipoPrecio);

                //Criterio 4: EscaladoKG
                CalcularCriterioAjusteEscaladoKg(parametros, precios, configCalculo, tipoPrecio);

                //Criterio 5: Area
                CalcularCriterioArea(parametros, precios, configCalculo, tipoPrecio);

                //Criterio 6: Comisionista
                CalcularCriterioComisionista(parametros, precios, configCalculo, tipoPrecio);
            }
        }

        private void AplicarCambioMoneda(Dictionary<short, decimal> precios, decimal monedaValorCambio)
        {
            foreach (short idEscaladoKg in ListaEscaladosKg.EscaladosKg)
            {
                if (precios.ContainsKey(idEscaladoKg))
                {
                    precios[idEscaladoKg] = precios[idEscaladoKg] * monedaValorCambio;
                }
            }
        }

        private void CalcularCriterioBase(ParametrosCalculoPrecio parametros, DatosPrecioCalculado precios, ConfiguracionCalculo configCalculo, short tipoPrecio)
        {
            foreach (short escaladoKg in ListaEscaladosKg.EscaladosKg)
            {
                int? idEscaladoCoste = configCalculo.ObtenerEscaladoCoste(precios.Coste[escaladoKg]);
                if ((idEscaladoCoste ?? 0) == 0)
                {
                    throw new ApplicationException(String.Format(Mensajes.NoExisteEscaladoCosteConfigurado, precios.Coste[escaladoKg]));
                }
                Calculo_Valores valorCalculo = configCalculo.ValoresBase.Where(v => v.IdEscaladoKg == escaladoKg && v.IdCalculo_EscaladoCoste == idEscaladoCoste && v.IdAplicacion == parametros.IdAplicacion && v.IdTipoPrecio == tipoPrecio).FirstOrDefault();
                decimal? valor = CalcularValorCriterioBase(parametros, precios.Coste[escaladoKg], valorCalculo);
                precios.AsignarPrecio(escaladoKg, valor, tipoPrecio);
            }
        }

        private void CalcularCriterioPolimero(ParametrosCalculoPrecio parametros, DatosPrecioCalculado precios, ConfiguracionCalculo configCalculo, short tipoPrecio)
        {
            if (configCalculo.ContieneCriterio((short)Criterios.Polimero))
            {
                foreach (short escaladoKg in ListaEscaladosKg.EscaladosKg)
                {
                    Calculo_Polimero valorPolimero = configCalculo.ValoresPolimeros.Where(v => v.IdEscaladoKg == escaladoKg && v.IdTipoPrecio == tipoPrecio).FirstOrDefault();
                    decimal? valor = CalcularValorPolimero(parametros, precios, escaladoKg, tipoPrecio, valorPolimero);
                    if (valor.HasValue)
                    {
                        precios.AsignarPrecio(escaladoKg, valor, tipoPrecio);
                    }
                }
            }
        }

        private void CalcularCriterioDistribuidor(ParametrosCalculoPrecio parametros, DatosPrecioCalculado precios, ConfiguracionCalculo configCalculo, short tipoPrecio)
        {
            if (configCalculo.ContieneCriterio((short)Criterios.Distribuidor))
            {
                precios.AplicarIncrementoPorcentaje(parametros.IncrementoNoDistribuidor, tipoPrecio);
            }
        }

        private void CalcularCriterioAjusteEscaladoKg(ParametrosCalculoPrecio parametros, DatosPrecioCalculado precios, ConfiguracionCalculo configCalculo, short tipoPrecio)
        {
            if (configCalculo.ContieneCriterio((short)Criterios.AjusteEscaladoKg))
            {
                if (!configCalculo.Calculo.IdEscaladoKgBase.HasValue)
                {
                    throw new ApplicationException(Mensajes.NoExisteEscaladoBaseAjusteKg);
                }

                short escaladoBase = configCalculo.Calculo.IdEscaladoKgBase.Value;
                short escaladoAnterior = escaladoBase;
                //Ajustar escalados menores
                foreach (short escaladoKg in ListaEscaladosKg.EscaladosKg.Where(e => e < escaladoBase).OrderByDescending(e => e))
                {
                    Calculo_EscaladoKg valorAjuste = configCalculo.ValoresAjusteEscaladoKg.Where(v => v.IdEscaladoKg == escaladoKg && v.IdAplicacion == parametros.IdAplicacion && v.IdTipoPrecio == tipoPrecio).FirstOrDefault();
                    decimal? valor = CalcularValorAjusteKg(parametros, precios, escaladoAnterior, tipoPrecio, valorAjuste);
                    precios.AsignarPrecio(escaladoKg, valor, tipoPrecio);

                    escaladoAnterior = escaladoKg;
                }
                //Ajustar escalados mayores
                escaladoAnterior = escaladoBase;
                foreach (short escaladoKg in ListaEscaladosKg.EscaladosKg.Where(e => e > escaladoBase).OrderBy(e => e))
                {
                    Calculo_EscaladoKg valorAjuste = configCalculo.ValoresAjusteEscaladoKg.Where(v => v.IdEscaladoKg == escaladoKg && v.IdTipoPrecio == tipoPrecio).FirstOrDefault();
                    decimal? valor = CalcularValorAjusteKg(parametros, precios, escaladoAnterior, tipoPrecio, valorAjuste);
                    precios.AsignarPrecio(escaladoKg, valor, tipoPrecio);

                    escaladoAnterior = escaladoKg;
                }
            }
        }
        private void CalcularCriterioArea(ParametrosCalculoPrecio parametros, DatosPrecioCalculado precios, ConfiguracionCalculo configCalculo, short tipoPrecio)
        {
            if (configCalculo.ContieneCriterio((short)Criterios.Area))
            {
                foreach (short escaladoKg in ListaEscaladosKg.EscaladosKg)
                {
                    Calculo_Area valorArea = configCalculo.ValoresAreas.Where(v => v.IdEscaladoKg == escaladoKg && v.IdTipoPrecio == tipoPrecio).FirstOrDefault();
                    decimal? valor = CalcularValorArea(parametros, precios, escaladoKg, tipoPrecio, valorArea);
                    if (valor.HasValue)
                    {
                        precios.AsignarPrecio(escaladoKg, valor, tipoPrecio);
                    }
                }
            }
        }
        private void CalcularCriterioComisionista(ParametrosCalculoPrecio parametros, DatosPrecioCalculado precios, ConfiguracionCalculo configCalculo, short tipoPrecio)
        {
            if (configCalculo.ContieneCriterio((short)Criterios.Comisionista))
            {
                precios.AplicarIncrementoPorcentaje(parametros.Comision, tipoPrecio);
            }
        }


        private decimal? CalcularValorArea(ParametrosCalculoPrecio parametros, DatosPrecioCalculado precios, short idEscaladoKg, short tipoPrecio, Calculo_Area valorArea)
        {
            decimal? valorActual = precios.ObtenerPrecio(idEscaladoKg, tipoPrecio);

            if (valorArea != null && valorActual.HasValue)
            {
                switch (valorArea.IdTipoValor)
                {
                    case (short)TiposValor.Incremento:
                        return valorActual + valorArea.Valor;
                    case (short)TiposValor.Porcentaje:
                        return valorActual + (valorActual * valorArea.Valor / 100);
                    default:
                        return null;
                }
            }

            return null;
        }

        private decimal? CalcularValorPolimero(ParametrosCalculoPrecio parametros, DatosPrecioCalculado precios, short idEscaladoKg, short tipoPrecio, Calculo_Polimero valorPolimero)
        {
            decimal? valorActual = precios.ObtenerPrecio(idEscaladoKg, tipoPrecio);

            if (valorPolimero != null && valorActual.HasValue)
            {
                switch (valorPolimero.IdTipoValor)
                {
                    case (short)TiposValor.Incremento:
                        return valorActual + valorPolimero.Valor;
                    case (short)TiposValor.Porcentaje:
                        return valorActual + (valorActual * valorPolimero.Valor / 100);
                    default:
                        return null;
                }
            }

            return null;
        }

        private decimal? CalcularValorAjusteKg(ParametrosCalculoPrecio parametros, DatosPrecioCalculado precios, short escaladoAnterior, short idTipoPrecio, Calculo_EscaladoKg valorAjuste)
        {
            decimal? ret = null;
            decimal? valorAnterior = precios.ObtenerPrecio(escaladoAnterior, idTipoPrecio);

            if (valorAjuste != null && valorAnterior.HasValue)
            {
                switch (valorAjuste.IdTipoValor)
                {
                    case (short)TiposValor.Incremento:
                        return valorAnterior + valorAjuste.Valor;
                    case (short)TiposValor.Porcentaje:
                        return valorAnterior + (valorAnterior * valorAjuste.Valor / 100);
                    default:
                        return null;
                }
            }

            return ret;
        }

        private decimal? CalcularValorCriterioBase(ParametrosCalculoPrecio parametros, decimal coste, Calculo_Valores valor)
        {
            decimal? ret = null;

            if(valor != null)
            {
                switch(valor.IdTipoValor)
                {
                    case (short)TiposValor.Incremento:
                        return coste + valor.Valor;
                    case (short)TiposValor.Porcentaje:
                        return coste + (coste * valor.Valor / 100);
                    case (short)TiposValor.ValorFinal:
                        return valor.Valor;
                    default:
                        return null;
                }
            }

            return ret;

        }
    }
}
